/* $Id: encrypt.cpp,v 1.9 2010/11/05 13:06:20 stm Exp $
 *
 * PDFlib PLOP: PDF Linearization, Optimization, Protection
 * encryption sample
 *
 */

#include <iostream>
#include <string>

#include "plop.hpp"

using namespace std;
using namespace pdflib;

int
main(void)
{
    PLOP plop;

    /* parameters for the input document */
    const wstring in_filename(L"PLOP-datasheet.pdf");
    const wstring in_password(L"");

    /* parameters for the output document */
    const wstring out_filename(L"PLOP-datasheet-encrypted.pdf");
    const wstring out_master(L"DEMO");
    const wstring out_user(L"demo");
    const wstring permissions(L"");

    /* This is where input files live. Adjust as necessary. */
    const wstring searchpath(L"../data");

    try {
	int doc;
	wstring optlist;

	optlist = L"searchpath={" + searchpath + L"}";
	plop.set_option(optlist);

	/* open unprotected input file without any password */
	optlist = L"password {" + in_password + L"} ";
	if ((doc = plop.open_document(in_filename, optlist)) == -1)
	{
	    wcerr << L"Error: " << plop.get_errmsg() << endl;
	    return 2;
	}

	/* create the output file with a password */
	optlist = L"masterpassword {" + out_master + L"} userpassword {" +
	    out_user + L"} permissions {" + permissions + L"}";

	if (plop.create_file(out_filename, optlist) == -1)
	{
	    wcerr << L"Error: " << plop.get_errmsg() << endl;
	    return 2;
	}

	/* close input and output files */
	plop.close_document(doc);
    }
    catch (PLOP::Exception &ex) {
	wcerr << L"PLOP exception occurred in encrypt:" << endl;
	wcerr << L"[" << ex.get_errnum() << L"] " << ex.get_apiname()
		<< L": " << ex.get_errmsg() << endl;
	return 2;
    }

    return 0;
}
