#!/usr/bin/perl
# $Id: hellosign.pl,v 1.14 2011/02/11 08:52:42 rjs Exp $
#
# PDFlib $plop: PDF Linearization, Optimization, Protection
# hellosign sample: generate a document in memory with PDFlib, and sign it
# with $plop
#

use PDFlib::PLOP 4.1;
use PDFlib::PDFlib 8.0;


my $doc;
my $font;
my $contents;

# This is where input files live. Adjust as necessary.
my $searchpath = "../data";

my $out_filename = "hellosign.pdf";

# alternatives for specifying the certificate
#
# Windows Certificate Store (Windows only):
#    "engine=mscapi digitalid={certstore={store=My subject={DEMO PLOP user 2048}}} "
# PFX certificate from file in search path:
#    "digitalid={filename=demo2048.pfx} "
# PKCS#12 certificate from file in search path:
#    "digitalid={filename=demo2048.p12} "

my $sign_opts = "sign={" .
	"appearance={fieldname=Signature1 } " .
	"digitalid={filename=demo2048.p12} " .
	"password={demo} " .
	"reason={I agree to this document} " .
	"contactinfo={tel (123) 456 789} " .
	"location=Munich " .
	"}";

# ------------- Step 1: generate the document in memory ------------
my $p = new PDFlib::PDFlib;

eval {
    # open new PDF file in memory

    $p->begin_document("", "");

    $p->set_info("Creator", "hellosign.c");
    $p->set_info("Author", "Rainer Plckl");
    $p->set_info("Title", "Hello, world/signed (C)!");

    $font = $p->load_font("Helvetica-Bold", "host", "");

    $p->begin_page_ext(595, 842, "");

    $p->setfont($font, 24);
    $p->set_text_pos(50, 700);
    $p->show("Hello, world/signed!");
    $p->continue_text("(says C)");
    $p->end_page_ext("");

    $p->end_document("");

    # fetch the buffer containing the PDF document
    $contents = $p->get_buffer();
};

if ($@) {
    die("$0: PDFlib Exception occurred:\n$@");
}

# ------------------ Step 2: sign the document -----------------
    
# create a new plop object
my $plop = new PDFlib::PLOP;

eval {

    $plop->set_option("searchpath={${searchpath}}");

    # Create a virtual PVF file from the data in memory
    $plop->create_pvf("/pvf/hello.pdf", $contents, "");

    # open unsigned input document from PVF file
    if (($doc = $plop->open_document("/pvf/hello.pdf", "")) == -1)
    {
	die("Error: %s\n", $plop->get_errmsg());
    }

    # sign the output file
    if ($plop->create_file($out_filename, $sign_opts) == -1)
    {
	die("Error: %s\n", $plop->get_errmsg());
    }

    # close the output file
    $plop->close_document($doc);


    # delete PVF file containing input document
    if ($plop->delete_pvf("/pvf/hello.pdf") == -1)
    {
	die("Error: %s\n", $plop->get_errmsg());
    }
};

if ($@) {
    die("$0: PLOP Exception occurred:\n$@");
}
