#!/usr/bin/perl
# $Id: hellosign.pl,v 1.3 2011/02/23 18:44:01 rjs Exp $
#
# PDFlib $plop: PDF Linearization, Optimization, Protection
# hellosign sample: generate a document in memory with PDFlib, and sign it
# with $plop
#

use plop_pl 4.1
use pdflib_pl 7.0;


my $doc;
my $font;
my $contents;

# This is where input files live. Adjust as necessary.
my $searchpath = "../data";

my $out_filename = "hellosign.pdf";

# alternatives for specifying the certificate
#
# Windows Certificate Store (Windows only):
#    "engine=mscapi digitalid={certstore={store=My subject={DEMO PLOP user 2048}}} "
# PFX certificate from file in search path:
#    "digitalid={filename=demo2048.pfx} "
# PKCS#12 certificate from file in search path:
#    "digitalid={filename=demo2048.p12} "

my $sign_opts = "sign={" .
	"appearance={fieldname=Signature1 } " .
	"digitalid={filename=demo2048.p12} " .
	"password={demo} " .
	"reason={I agree to this document} " .
	"contactinfo={tel (123) 456 789} " .
	"location=Munich " .
	"}";

# ------------- Step 1: generate the document in memory ------------
my $p = PDF_new();


eval {
    # open new PDF file in memory

    PDF_begin_document($p, "", "");

    PDF_set_info($p, "Creator", "hellosign.c");
    PDF_set_info($p, "Author", "Rainer Plckl");
    PDF_set_info($p, "Title", "Hello, world/signed (C)!");

    $font = PDF_load_font($p, "Helvetica-Bold", "host", "");

    PDF_begin_page_ext($p, 595, 842, "");

    PDF_setfont($p, $font, 24);
    PDF_set_text_pos($p, 50, 700);
    PDF_show($p, "Hello, world/signed!");
    PDF_continue_text($p, "(says C)");
    PDF_end_page_ext($p, "");

    PDF_end_document($p, "");

    # fetch the buffer containing the PDF document
    $contents = PDF_get_buffer($p);
};

if ($@) {
    printf("PDFlib exception occurred in hello sample:\n");
    printf(" $@\n");
    PDF_delete($p);
    exit(98);
}

# ------------------ Step 2: sign the document -----------------
    
# create a new plop object
my $plop = PLOP_new();

eval {

    PLOP_set_option($plop, "searchpath={${searchpath}}");

    # Create a virtual PVF file from the data in memory
    PLOP_create_pvf($plop, "/pvf/hello.pdf", $contents, "");

    # open unsigned input document from PVF file
    if (($doc = PLOP_open_document($plop, "/pvf/hello.pdf", "")) == -1)
    {
	printf("Error: %s\n", PLOP_get_errmsg($plop));
	PLOP_delete($plop);
	PDF_delete($p);
	exit(99);
    }

    # sign the output file
    if (PLOP_create_file($plop, $out_filename, $sign_opts) == -1)
    {
	printf("Error: %s\n", PLOP_get_errmsg($plop));
	PLOP_delete($plop);
	PDF_delete($p);
	exit(99);
    }

    # close the output file
    PLOP_close_document($plop, $doc);


    # delete PVF file containing input document
    if (PLOP_delete_pvf($plop, "/pvf/hello.pdf") == -1 )
    {
	printf("Error: %s\n", PLOP_get_errmsg($plop));
	PLOP_delete($plop);
	PDF_delete($p);
	exit(99);
    }
};

if ($@) {
    printf("PLOP exception occurred in hellosign:\n");
    printf(" $@\n");
    PLOP_delete($plop);
    exit(99);
}

# delete the $plop object
PLOP_delete($plop);

# delete the PDF object
PDF_delete($p);
