<?php
# $Id: hellosign.php,v 1.11 2011/02/23 18:51:35 rjs Exp $
#
# PDFlib PLOP: PDF Linearization, Optimization, Protection
# hellosign sample: generate a document in memory with PDFlib, and sign it
# with PLOP
#

# This is where input files live. Adjust as necessary.
$searchpath = "../../data ../../data";

$out_filename = "hellosign.pdf";

# alternatives for specifying the certificate
#
# Windows Certificate Store (Windows only):
#    "engine=mscapi digitalid={certstore={store=My subject={DEMO PLOP user 2048}}} "
# PFX certificate from file in search path:
#    "digitalid={filename=demo2048.pfx} "
# PKCS#12 certificate from file in search path:
#     "digitalid={filename=demo2048.p12} "

$sign_opts = "sign={" .
	"appearance={fieldname=Signature1 } " .
	"digitalid={filename=demo2048.p12} " .
	"password={demo} " .
	"reason={I agree to this document} " .
	"contactinfo={tel (123) 456 789} " .
	"location=Munich " .
	"}";

# ------------- Step 1: generate the document in memory ------------
try {
    $p = new PDFlib();

    # open new PDF file in memory

    $p->begin_document("", "");

    $p->set_info("Creator", "hellosign.c");
    $p->set_info("Author", "Rainer Plckl");
    $p->set_info("Title", "Hello, world/signed (C)!");

    $font = $p->load_font("Helvetica-Bold", "host", "");

    $p->begin_page_ext(595, 842, "");

    $p->setfont($font, 24);
    $p->set_text_pos(50, 700);
    $p->show("Hello, world/signed!");
    $p->continue_text("(says C)");
    $p->end_page_ext("");

    $p->end_document("");

    # fetch the buffer containing the PDF document
    $contents = $p->get_buffer();

# ------------------ Step 2: sign the document -----------------
    
    $plop = new PLOP();


    $plop->set_option("searchpath={${searchpath}}");

    # Create a virtual PVF file from the data in memory
    $plop->create_pvf("/pvf/hello.pdf", $contents, "");

    # open unsigned input document from PVF file
    if (($doc = $plop->open_document("/pvf/hello.pdf", "")) == 0)
    {
	die("Error: %s\n", $plop->get_errmsg());
    }

    # sign the output file
    if ($plop->create_file($out_filename, $sign_opts) == 0)
    {
	die("Error: %s\n", $plop->get_errmsg());
    }

    # close the output file
    $plop->close_document($doc);

    # delete PVF file containing input document
    if ($plop->delete_pvf("/pvf/hello.pdf") == 0)
    {
	die("Error: %s\n", $plop->get_errmsg());
    }

}
catch (PLOPException $e) {
    die("PLOP exception occurred in hellosign sample:\n" .
	"[" . $e->get_errnum() . "] " . $e->get_apiname() . ": " .
	$e->get_errmsg() . "\n");
    }
catch (Exception $e) {
    die($e);
}

$p = 0;
header("text/html");
print "<a href=\"$out_filename\">click here</a> to open the signed PDF file";
?>
